@@ -87,16 +87,39 @@ def _iter_child_nodes_in_order_internal_1(node):
         yield list(zip(node.keys, node.values))
     elif isinstance(node, ast.FunctionDef):
         if six.PY2:
-            assert node._fields == ('name', 'args', 'body', 'decorator_list')
+            assert node._fields == ('name', 'args', 'body', 'decorator_list'), node._fields
+            yield node.decorator_list, node.args, node.body
+        elif sys.version_info >= (3, 8):
+            assert node._fields == (
+                "name",
+                "args",
+                "body",
+                "decorator_list",
+                "returns",
+                "type_comment",
+            ), node._fields
+            res = (
+                node.type_comment,
+                node.decorator_list,
+                node.args,
+                node.returns,
+                node.body,
+            )
+            yield res
         else:
-            assert node._fields == ('name', 'args', 'body', 'decorator_list', 'returns')
-        yield node.decorator_list, node.args, node.body
+            assert node._fields == ('name', 'args', 'body', 'decorator_list',
+                                    'returns'), node._fields
+            yield node.decorator_list, node.args, node.returns, node.body
         # node.name is a string, not an AST node
     elif isinstance(node, ast.arguments):
         if six.PY2:
-            assert node._fields == ('args', 'vararg', 'kwarg', 'defaults')
+            assert node._fields == ('args', 'vararg', 'kwarg', 'defaults'), node._fields
+        elif sys.version_info >= (3, 8):
+            assert node._fields == ('posonlyargs', 'args', 'vararg', 'kwonlyargs',
+                                    'kw_defaults', 'kwarg', 'defaults'), node._fields
         else:
-            assert node._fields == ('args', 'vararg', 'kwonlyargs', 'kw_defaults', 'kwarg', 'defaults')
+            assert node._fields == ('args', 'vararg', 'kwonlyargs',
+                                    'kw_defaults', 'kwarg', 'defaults'), node._fields
         defaults = node.defaults or ()
         num_no_default = len(node.args)-len(defaults)
         yield node.args[:num_no_default]
@@ -105,6 +128,12 @@ def _iter_child_nodes_in_order_internal_1(node):
     elif isinstance(node, ast.IfExp):
         assert node._fields == ('test', 'body', 'orelse')
         yield node.body, node.test, node.orelse
+    elif isinstance(node, ast.Call):
+        # call arguments order are lost by ast, re-order them
+        yield node.func
+        args = sorted([(k.value.lineno, k.value.col_offset, k) for k in node.keywords]+
+                      [(k.lineno,k.col_offset, k) for k in node.args])
+        yield [a[2] for a in args]
     elif isinstance(node, ast.ClassDef):
         if six.PY2:
             assert node._fields == ('name', 'bases', 'body', 'decorator_list')
@@ -112,6 +141,9 @@ def _iter_child_nodes_in_order_internal_1(node):
             assert node._fields == ('name', 'bases', 'keywords', 'body', 'decorator_list')
         yield node.decorator_list, node.bases, node.body
         # node.name is a string, not an AST node
+    elif sys.version_info >= (3, 6) and isinstance(node, ast.FormattedValue):
+        assert node._fields == ('value', 'conversion', 'format_spec')
+        yield node.value,
     else:
         # Default behavior.
         yield ast.iter_child_nodes(node)
@@ -143,6 +175,11 @@ def _flags_to_try(source, flags, auto_flags, mode):
     If ``auto_flags`` is True, then yield ``flags`` and ``flags ^ print_function``.
     """
     flags = CompilerFlags(flags)
+    if sys.version_info >= (3, 8):
+        if re.search(r"# *type:", source):
+            flags = flags | CompilerFlags('type_comments')
+        yield flags
+        return
     if not auto_flags:
         yield flags
         return
@@ -181,7 +218,6 @@ def _parse_ast_nodes(text, flags, auto_flags, mode):
       ``ast.Module``
     """
     text = FileText(text)
-    flags = CompilerFlags(flags)
     filename = str(text.filename) if text.filename else "<unknown>"
     source = text.joined
     source = dedent(source)
@@ -215,7 +251,7 @@ def _test_parse_string_literal(text, flags):
     Attempt to parse ``text``.  If it parses cleanly to a single string
     literal, return its value.  Otherwise return ``None``.
 
-      >>> _test_parse_string_literal(r'"foo\n" r"\nbar"', 0)
+      >>> _test_parse_string_literal(r'"foo\n" r"\nbar"', None)
       'foo\n\\nbar'
 
     """
@@ -309,6 +345,18 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
     :raise ValueError:
       Could not find the starting line number.
     """
+    # joined strings and children do not carry a column offset on pre-3.8
+    # this prevent reformatting.
+    # set the column offset to the parent value before 3.8
+    if (3, 6) < sys.version_info < (3, 8):
+        if (
+            isinstance(ast_node, getattr(ast, "JoinedStr", None))
+            or isinstance(
+                parent_ast_node, (getattr(ast, "JoinedStr", None), ast.FormattedValue)
+            )
+        ) and ast_node.col_offset == -1:
+            ast_node.col_offset = parent_ast_node.col_offset
+
     # First, traverse child nodes.  If the first child node (recursively) is a
     # multiline string, then we need to transfer its information to this node.
     # Walk all nodes/fields of the AST.  We implement this as a custom
@@ -344,17 +392,29 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
                     "probably the handler for ast.%s." % type(ast_node).__name__)
             child_minpos = child_node.startpos
         is_first_child = False
+
     # If the node has no lineno at all, then skip it.  This should only happen
     # for nodes we don't care about, e.g. ``ast.Module`` or ``ast.alias``.
     if not hasattr(ast_node, 'lineno'):
         return False
     # If col_offset is set then the lineno should be correct also.
     if ast_node.col_offset >= 0:
+        # In Python 3.8+, FunctionDef.lineno is the line with the def. To
+        # account for decorators, we need the lineno of the first decorator
+        if (sys.version_info >= (3, 8)
+            and isinstance(ast_node, (ast.FunctionDef, ast.ClassDef))
+            and ast_node.decorator_list):
+            delta = (ast_node.decorator_list[0].lineno-1,
+                     # The col_offset doesn't include the @
+                     ast_node.decorator_list[0].col_offset - 1)
+        else:
+            delta = (ast_node.lineno-1, ast_node.col_offset)
+
         # Not a multiline string literal.  (I.e., it could be a non-string or
         # a single-line string.)
         # Easy.
-        delta = (ast_node.lineno-1, ast_node.col_offset)
         startpos = text.startpos + delta
+
         # Special case for 'with' statements.  Consider the code:
         #    with X: pass
         #    ^0   ^5
@@ -387,6 +447,7 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             assert str(text[startpos:(startpos+(0,4))]) == "with"
         ast_node.startpos = startpos
         return False
+
     assert ast_node.col_offset == -1
     if leftstr_node:
         # This is an ast node where the leftmost deepest leaf is a
@@ -411,6 +472,7 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
         assert leftstr_node.col_offset == -1
         ast_node.startpos = leftstr_node.startpos
         return True
+
     # It should now be the case that we are looking at a multi-line string
     # literal.
     if not isinstance(ast_node, (ast.Str, Bytes)):
@@ -435,6 +497,7 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             (_m.group()[-1], FilePos(start_lineno, _m.start()+start_line_colno))
             for _m in re.finditer("[bBrRuU]*[\"\']", start_line)])
     target_str = ast_node.s
+
     # Loop over possible end_linenos.  The first one we've identified is the
     # by far most likely one, but in theory it could be anywhere later in the
     # file.  This could be because of a dastardly concatenated string like
@@ -700,7 +763,7 @@ class PythonStatement(object):
     Representation of a top-level Python statement or consecutive
     comments/blank lines.
 
-      >>> PythonStatement('print("x",\n file=None)\n', flags=0x10000)
+      >>> PythonStatement('print("x",\n file=None)\n', flags='print_function')  #doctest: +SKIP
       PythonStatement('print("x",\n file=None)\n', flags=0x10000)
 
     Implemented as a wrapper around a `PythonBlock` containing at most one
